"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const imperative_1 = require("@zowe/imperative");
const path = require("path");
const url_1 = require("url");
const vscode = require("vscode");
const zowe = require("@zowe/cli");
const ZoweExplorerApiRegister_1 = require("./api/ZoweExplorerApiRegister");
const utils_1 = require("./utils");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
let IConnection;
let InputBoxOptions;
var ValidProfileEnum;
(function (ValidProfileEnum) {
    ValidProfileEnum[ValidProfileEnum["VALID"] = 0] = "VALID";
    ValidProfileEnum[ValidProfileEnum["INVALID"] = -1] = "INVALID";
})(ValidProfileEnum = exports.ValidProfileEnum || (exports.ValidProfileEnum = {}));
class Profiles {
    constructor(log) {
        this.log = log;
        this.allProfiles = [];
        this.validProfile = ValidProfileEnum.INVALID;
        this.dsSchema = "Zowe-DS-Persistent";
        this.ussSchema = "Zowe-USS-Persistent";
        this.jobsSchema = "Zowe-Jobs-Persistent";
        this.profilesByType = new Map();
        this.defaultProfileByType = new Map();
        this.profileManagerByType = new Map();
    }
    // Processing stops if there are no profiles detected
    static createInstance(log) {
        return __awaiter(this, void 0, void 0, function* () {
            Profiles.loader = new Profiles(log);
            yield Profiles.loader.refresh();
            return Profiles.loader;
        });
    }
    static getInstance() {
        return Profiles.loader;
    }
    checkCurrentProfile(theProfile) {
        return __awaiter(this, void 0, void 0, function* () {
            if ((!theProfile.profile.user) || (!theProfile.profile.password)) {
                try {
                    const values = yield Profiles.getInstance().promptCredentials(theProfile.name);
                    if (values !== undefined) {
                        this.usrNme = values[0];
                        this.passWrd = values[1];
                        this.baseEncd = values[2];
                    }
                }
                catch (error) {
                    utils_1.errorHandling(error, theProfile.name, localize(0, null) + `createUSSNodeDialog.optionalProfiles!`);
                    return;
                }
                if (this.usrNme !== undefined && this.passWrd !== undefined && this.baseEncd !== undefined) {
                    theProfile.profile.user = this.usrNme;
                    theProfile.profile.password = this.passWrd;
                    theProfile.profile.base64EncodedAuth = this.baseEncd;
                    this.validProfile = ValidProfileEnum.VALID;
                }
                else {
                    return;
                }
            }
            else {
                this.validProfile = ValidProfileEnum.VALID;
            }
        });
    }
    loadNamedProfile(name, type) {
        for (const profile of this.allProfiles) {
            if (profile.name === name && (type ? profile.type === type : true)) {
                return profile;
            }
        }
        throw new Error(localize(1, null)
            + name + localize(2, null));
    }
    getDefaultProfile(type = "zosmf") {
        return this.defaultProfileByType.get(type);
    }
    getProfiles(type = "zosmf") {
        return this.profilesByType.get(type);
    }
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            this.allProfiles = [];
            this.allTypes = [];
            for (const type of ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getInstance().registeredApiTypes()) {
                const profileManager = yield this.getCliProfileManager(type);
                const profilesForType = (yield profileManager.loadAll()).filter((profile) => {
                    return profile.type === type;
                });
                if (profilesForType && profilesForType.length > 0) {
                    this.allProfiles.push(...profilesForType);
                    this.profilesByType.set(type, profilesForType);
                    let defaultProfile;
                    try {
                        defaultProfile = yield profileManager.load({ loadDefault: true });
                    }
                    catch (error) {
                        vscode.window.showInformationMessage(error.message);
                    }
                    this.defaultProfileByType.set(type, defaultProfile);
                }
                // This is in the loop because I need an instantiated profile manager config
                if (profileManager.configurations && this.allTypes.length === 0) {
                    for (const element of profileManager.configurations) {
                        this.allTypes.push(element.type);
                    }
                }
            }
        });
    }
    validateAndParseUrl(newUrl) {
        let url;
        const validProtocols = ["https"];
        const DEFAULT_HTTPS_PORT = 443;
        const validationResult = {
            valid: false,
            host: null,
            port: null
        };
        try {
            url = new url_1.URL(newUrl);
        }
        catch (error) {
            return validationResult;
        }
        // overkill with only one valid protocol, but we may expand profile types and protocols in the future?
        if (!validProtocols.some((validProtocol) => url.protocol.includes(validProtocol))) {
            return validationResult;
        }
        // if port is empty, set https defaults
        if (!url.port.trim()) {
            validationResult.port = DEFAULT_HTTPS_PORT;
        }
        else {
            validationResult.port = Number(url.port);
        }
        validationResult.host = url.hostname;
        validationResult.valid = true;
        return validationResult;
    }
    getUrl(urlInputBox) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                urlInputBox.onDidHide(() => { resolve(urlInputBox.value); });
                urlInputBox.onDidAccept(() => {
                    if (this.validateAndParseUrl(urlInputBox.value).valid) {
                        resolve(urlInputBox.value);
                    }
                    else {
                        urlInputBox.validationMessage = localize(3, null);
                    }
                });
            });
        });
    }
    editSession(profileLoaded, profileName) {
        return __awaiter(this, void 0, void 0, function* () {
            const editSession = profileLoaded.profile;
            const editURL = "https://" + editSession.host + ":" + editSession.port;
            const editUser = editSession.user;
            const editPass = editSession.password;
            const editrej = editSession.rejectUnauthorized;
            let updUser;
            let updPass;
            let updRU;
            const updUrl = yield this.urlInfo(editURL);
            if (updUrl) {
                updUser = yield this.userInfo(editUser);
            }
            else {
                return;
            }
            if (updUser !== undefined) {
                updPass = yield this.passwordInfo(editPass);
            }
            else {
                return;
            }
            if (updPass !== undefined) {
                updRU = yield this.ruInfo(editrej);
            }
            else {
                return;
            }
            if (updRU !== undefined) {
                const updProfile = {
                    host: updUrl.host,
                    port: updUrl.port,
                    user: updUser,
                    password: updPass,
                    rejectUnauthorized: updRU,
                    base64EncodedAuth: null
                };
                try {
                    const updSession = yield zowe.ZosmfSession.createBasicZosmfSession(updProfile);
                    updProfile.base64EncodedAuth = updSession.ISession.base64EncodedAuth;
                    yield this.updateProfile({ profile: updProfile, name: profileName });
                    vscode.window.showInformationMessage(localize(4, null));
                    return updProfile;
                }
                catch (error) {
                    yield utils_1.errorHandling(error.message);
                }
            }
        });
    }
    createNewConnection(profileName, profileType = "zosmf") {
        return __awaiter(this, void 0, void 0, function* () {
            let newUser;
            let newPass;
            let newRU;
            const newUrl = yield this.urlInfo();
            if (newUrl) {
                newUser = yield this.userInfo();
            }
            else {
                return;
            }
            if (newUser !== undefined) {
                newPass = yield this.passwordInfo();
            }
            else {
                return;
            }
            if (newPass !== undefined) {
                newRU = yield this.ruInfo();
            }
            else {
                return;
            }
            if (newRU !== undefined) {
                for (const profile of this.allProfiles) {
                    if (profile.name === profileName) {
                        vscode.window.showErrorMessage(localize(5, null));
                        return undefined;
                    }
                }
                IConnection = {
                    name: profileName,
                    host: newUrl.host,
                    port: newUrl.port,
                    user: newUser,
                    password: newPass,
                    rejectUnauthorized: newRU
                };
                try {
                    yield zowe.ZosmfSession.createBasicZosmfSession(IConnection);
                    yield this.saveProfile(IConnection, IConnection.name, profileType);
                    vscode.window.showInformationMessage("Profile " + profileName + " was created.");
                    return profileName;
                }
                catch (error) {
                    yield utils_1.errorHandling(error.message);
                }
            }
            else {
                return;
            }
        });
    }
    promptCredentials(sessName, rePrompt) {
        return __awaiter(this, void 0, void 0, function* () {
            let repromptUser;
            let repromptPass;
            let loadProfile;
            let loadSession;
            let newUser;
            let newPass;
            try {
                loadProfile = this.loadNamedProfile(sessName.trim());
                loadSession = loadProfile.profile;
            }
            catch (error) {
                yield utils_1.errorHandling(error.message);
            }
            if (rePrompt) {
                repromptUser = loadSession.user;
                repromptPass = loadSession.password;
            }
            if (!loadSession.user || rePrompt) {
                newUser = yield this.userInfo(repromptUser);
                loadSession.user = loadProfile.profile.user = newUser;
            }
            if (newUser === undefined) {
                return;
            }
            else {
                if (!loadSession.password || rePrompt) {
                    newPass = yield this.passwordInfo(repromptPass);
                    loadSession.password = loadProfile.profile.password = newPass;
                }
            }
            if (newPass === undefined) {
                return;
            }
            else {
                try {
                    const updSession = yield zowe.ZosmfSession.createBasicZosmfSession(loadSession);
                    if (rePrompt) {
                        yield this.updateProfile(loadProfile, rePrompt);
                    }
                    return [updSession.ISession.user, updSession.ISession.password, updSession.ISession.base64EncodedAuth];
                }
                catch (error) {
                    yield utils_1.errorHandling(error.message);
                }
            }
        });
    }
    getDeleteProfile() {
        return __awaiter(this, void 0, void 0, function* () {
            const allProfiles = this.allProfiles;
            const profileNamesList = allProfiles.map((temprofile) => {
                return temprofile.name;
            });
            if (!profileNamesList.length) {
                vscode.window.showInformationMessage(localize(6, null));
                return;
            }
            const quickPickList = {
                placeHolder: localize(7, null),
                ignoreFocusOut: true,
                canPickMany: false
            };
            const sesName = yield vscode.window.showQuickPick(profileNamesList, quickPickList);
            if (sesName === undefined) {
                vscode.window.showInformationMessage(localize(8, null));
                return;
            }
            return allProfiles.find((temprofile) => temprofile.name === sesName);
        });
    }
    deletePrompt(deletedProfile) {
        return __awaiter(this, void 0, void 0, function* () {
            const profileName = deletedProfile.name;
            this.log.debug(localize(9, null) + profileName);
            const quickPickOptions = {
                placeHolder: localize(10, null) + profileName,
                ignoreFocusOut: true,
                canPickMany: false
            };
            // confirm that the user really wants to delete
            if ((yield vscode.window.showQuickPick([localize(11, null),
                localize(12, null)], quickPickOptions)) !== localize(13, null)) {
                this.log.debug(localize(14, null));
                return;
            }
            const profileType = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(deletedProfile).getProfileTypeName();
            try {
                this.deleteProf(deletedProfile, profileName, profileType);
            }
            catch (error) {
                this.log.error(localize(15, null) + JSON.stringify(error));
                yield utils_1.errorHandling(error, profileName, error.message);
                throw error;
            }
            vscode.window.showInformationMessage("Profile " + profileName + " was deleted.");
            return profileName;
        });
    }
    deleteProf(ProfileInfo, ProfileName, ProfileType) {
        return __awaiter(this, void 0, void 0, function* () {
            let zosmfProfile;
            try {
                zosmfProfile = yield (yield this.getCliProfileManager(ProfileType))
                    .delete({ profile: ProfileInfo, name: ProfileName, type: ProfileType });
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
            return zosmfProfile.profile;
        });
    }
    deleteProfile(datasetTree, ussTree, jobsProvider, node) {
        return __awaiter(this, void 0, void 0, function* () {
            let deleteLabel;
            let deletedProfile;
            if (!node) {
                deletedProfile = yield this.getDeleteProfile();
            }
            else {
                deletedProfile = node.getProfile();
            }
            if (!deletedProfile) {
                return;
            }
            deleteLabel = deletedProfile.name;
            const deleteSuccess = yield this.deletePrompt(deletedProfile);
            if (!deleteSuccess) {
                vscode.window.showInformationMessage(localize(16, null));
                return;
            }
            // Delete from Data Set Recall
            const recallDs = datasetTree.getRecall();
            recallDs.slice().reverse()
                .filter((ds) => ds.substring(1, ds.indexOf("]")).trim() === deleteLabel.toUpperCase())
                .forEach((ds) => {
                datasetTree.removeRecall(ds);
            });
            // Delete from Data Set Favorites
            const favoriteDs = datasetTree.mFavorites;
            for (let i = favoriteDs.length - 1; i >= 0; i--) {
                const findNode = favoriteDs[i].label.substring(1, favoriteDs[i].label.indexOf("]")).trim();
                if (findNode === deleteLabel) {
                    datasetTree.removeFavorite(favoriteDs[i]);
                    favoriteDs[i].dirty = true;
                    datasetTree.refresh();
                }
            }
            // Delete from Data Set Tree
            datasetTree.mSessionNodes.forEach((sessNode) => {
                if (sessNode.getProfileName() === deleteLabel) {
                    datasetTree.deleteSession(sessNode);
                    sessNode.dirty = true;
                    datasetTree.refresh();
                }
            });
            // Delete from USS Recall
            const recallUSS = ussTree.getRecall();
            recallUSS.slice().reverse()
                .filter((uss) => uss.substring(1, uss.indexOf("]")).trim() === deleteLabel.toUpperCase())
                .forEach((uss) => {
                ussTree.removeRecall(uss);
            });
            // Delete from USS Favorites
            ussTree.mFavorites.forEach((ses) => {
                const findNode = ses.label.substring(1, ses.label.indexOf("]")).trim();
                if (findNode === deleteLabel) {
                    ussTree.removeFavorite(ses);
                    ses.dirty = true;
                    ussTree.refresh();
                }
            });
            // Delete from USS Tree
            ussTree.mSessionNodes.forEach((sessNode) => {
                if (sessNode.getProfileName() === deleteLabel) {
                    ussTree.deleteSession(sessNode);
                    sessNode.dirty = true;
                    ussTree.refresh();
                }
            });
            // Delete from Jobs Favorites
            jobsProvider.mFavorites.forEach((ses) => {
                const findNode = ses.label.substring(1, ses.label.indexOf("]")).trim();
                if (findNode === deleteLabel) {
                    jobsProvider.removeFavorite(ses);
                    ses.dirty = true;
                    jobsProvider.refresh();
                }
            });
            // Delete from Jobs Tree
            jobsProvider.mSessionNodes.forEach((jobNode) => {
                if (jobNode.getProfileName() === deleteLabel) {
                    jobsProvider.deleteSession(jobNode);
                    jobNode.dirty = true;
                    jobsProvider.refresh();
                }
            });
            // Delete from Data Set Sessions list
            const dsSetting = Object.assign({}, vscode.workspace.getConfiguration().get(this.dsSchema));
            let sessDS = dsSetting.sessions;
            let faveDS = dsSetting.favorites;
            sessDS = sessDS.filter((element) => {
                return element.trim() !== deleteLabel;
            });
            faveDS = faveDS.filter((element) => {
                return element.substring(1, element.indexOf("]")).trim() !== deleteLabel;
            });
            dsSetting.sessions = sessDS;
            dsSetting.favorites = faveDS;
            yield vscode.workspace.getConfiguration().update(this.dsSchema, dsSetting, vscode.ConfigurationTarget.Global);
            // Delete from USS Sessions list
            const ussSetting = Object.assign({}, vscode.workspace.getConfiguration().get(this.ussSchema));
            let sessUSS = ussSetting.sessions;
            let faveUSS = ussSetting.favorites;
            sessUSS = sessUSS.filter((element) => {
                return element.trim() !== deleteLabel;
            });
            faveUSS = faveUSS.filter((element) => {
                return element.substring(1, element.indexOf("]")).trim() !== deleteLabel;
            });
            ussSetting.sessions = sessUSS;
            ussSetting.favorites = faveUSS;
            yield vscode.workspace.getConfiguration().update(this.ussSchema, ussSetting, vscode.ConfigurationTarget.Global);
            // Delete from Jobs Sessions list
            const jobsSetting = Object.assign({}, vscode.workspace.getConfiguration().get(this.jobsSchema));
            let sessJobs = jobsSetting.sessions;
            let faveJobs = jobsSetting.favorites;
            sessJobs = sessJobs.filter((element) => {
                return element.trim() !== deleteLabel;
            });
            faveJobs = faveJobs.filter((element) => {
                return element.substring(1, element.indexOf("]")).trim() !== deleteLabel;
            });
            jobsSetting.sessions = sessJobs;
            jobsSetting.favorites = faveJobs;
            yield vscode.workspace.getConfiguration().update(this.jobsSchema, jobsSetting, vscode.ConfigurationTarget.Global);
            // Remove from list of all profiles
            const index = this.allProfiles.findIndex((deleteItem) => {
                return deleteItem === deletedProfile;
            });
            if (index >= 0) {
                this.allProfiles.splice(index, 1);
            }
        });
    }
    getAllTypes() {
        return this.allTypes;
    }
    getNamesForType(type) {
        return __awaiter(this, void 0, void 0, function* () {
            const profileManager = yield this.getCliProfileManager(type);
            const profilesForType = (yield profileManager.loadAll()).filter((profile) => {
                return profile.type === type;
            });
            return profilesForType.map((profile) => {
                return profile.name;
            });
        });
    }
    directLoad(type, name) {
        return __awaiter(this, void 0, void 0, function* () {
            let directProfile;
            const profileManager = yield this.getCliProfileManager(type);
            if (profileManager) {
                directProfile = yield profileManager.load({ name });
            }
            return directProfile;
        });
    }
    // ** Functions for handling Profile Information */
    urlInfo(input) {
        return __awaiter(this, void 0, void 0, function* () {
            let zosmfURL;
            const urlInputBox = vscode.window.createInputBox();
            if (input) {
                urlInputBox.value = input;
            }
            urlInputBox.ignoreFocusOut = true;
            urlInputBox.placeholder = localize(17, null);
            urlInputBox.prompt = localize(18, null);
            urlInputBox.show();
            zosmfURL = yield this.getUrl(urlInputBox);
            urlInputBox.dispose();
            if (!zosmfURL) {
                vscode.window.showInformationMessage(localize(19, null));
                return undefined;
            }
            const zosmfUrlParsed = this.validateAndParseUrl(zosmfURL);
            return zosmfUrlParsed;
        });
    }
    userInfo(input) {
        return __awaiter(this, void 0, void 0, function* () {
            let userName;
            if (input) {
                userName = input;
            }
            InputBoxOptions = {
                placeHolder: localize(20, null),
                prompt: localize(21, null),
                value: userName
            };
            userName = yield vscode.window.showInputBox(InputBoxOptions);
            if (userName === undefined) {
                vscode.window.showInformationMessage(localize(22, null));
                return undefined;
            }
            return userName;
        });
    }
    passwordInfo(input) {
        return __awaiter(this, void 0, void 0, function* () {
            let passWord;
            if (input) {
                passWord = input;
            }
            InputBoxOptions = {
                placeHolder: localize(23, null),
                prompt: localize(24, null),
                password: true,
                value: passWord
            };
            passWord = yield vscode.window.showInputBox(InputBoxOptions);
            if (passWord === undefined) {
                vscode.window.showInformationMessage(localize(25, null));
                return undefined;
            }
            return passWord;
        });
    }
    ruInfo(input) {
        return __awaiter(this, void 0, void 0, function* () {
            let rejectUnauthorize;
            if (input !== undefined) {
                rejectUnauthorize = input;
            }
            const quickPickOptions = {
                placeHolder: localize(26, null),
                ignoreFocusOut: true,
                canPickMany: false
            };
            const selectRU = ["True - Reject connections with self-signed certificates",
                "False - Accept connections with self-signed certificates"];
            const ruOptions = Array.from(selectRU);
            const chosenRU = yield vscode.window.showQuickPick(ruOptions, quickPickOptions);
            if (chosenRU === ruOptions[0]) {
                rejectUnauthorize = true;
            }
            else if (chosenRU === ruOptions[1]) {
                rejectUnauthorize = false;
            }
            else {
                vscode.window.showInformationMessage(localize(27, null));
                return undefined;
            }
            return rejectUnauthorize;
        });
    }
    // ** Functions that Calls Get CLI Profile Manager  */
    updateProfile(ProfileInfo, rePrompt) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const type of ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getInstance().registeredApiTypes()) {
                const profileManager = yield this.getCliProfileManager(type);
                this.loadedProfile = (yield profileManager.load({ name: ProfileInfo.name }));
            }
            const OrigProfileInfo = this.loadedProfile.profile;
            const NewProfileInfo = ProfileInfo.profile;
            if (!rePrompt) {
                OrigProfileInfo.user = NewProfileInfo.user;
                OrigProfileInfo.password = NewProfileInfo.password;
            }
            OrigProfileInfo.host = NewProfileInfo.host;
            OrigProfileInfo.port = NewProfileInfo.port;
            OrigProfileInfo.rejectUnauthorized = NewProfileInfo.rejectUnauthorized;
            const updateParms = {
                name: this.loadedProfile.name,
                merge: true,
                profile: OrigProfileInfo
            };
            try {
                (yield this.getCliProfileManager(this.loadedProfile.type)).update(updateParms);
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
        });
    }
    saveProfile(ProfileInfo, ProfileName, ProfileType) {
        return __awaiter(this, void 0, void 0, function* () {
            let zosmfProfile;
            try {
                zosmfProfile = yield (yield this.getCliProfileManager(ProfileType)).save({ profile: ProfileInfo, name: ProfileName, type: ProfileType });
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
            return zosmfProfile.profile;
        });
    }
    getCliProfileManager(type) {
        return __awaiter(this, void 0, void 0, function* () {
            let profileManager = this.profileManagerByType.get(type);
            if (!profileManager) {
                profileManager = yield new imperative_1.CliProfileManager({
                    profileRootDirectory: path.join(utils_1.getZoweDir(), "profiles"),
                    type
                });
                if (profileManager) {
                    this.profileManagerByType.set(type, profileManager);
                }
                else {
                    return undefined;
                }
            }
            return profileManager;
        });
    }
}
exports.Profiles = Profiles;

//# sourceMappingURL=Profiles.js.map
